
// ________________________________________________________________________
// 	file: ststem_controller
//	author: Ajay Dubey 	
// ________________________________________________________________________
// altera message_level Level1 
// altera message_off 10034 10035 10036 10037 10230 10240 10030 

 module master_slave_bridge 
	#(parameter BASE = 0)	// for simulation help only
    (
	input wire reset_n,
	input wire master_if_clk,
	input wire master_if_blksel,
	input wire master_if_write,
	input wire master_if_read,
	input wire [23:0] master_if_address,
	input wire [31:0] master_if_writedata,
	output wire[31:0] master_if_readdata,
	output wire master_if_readdata_valid,
	output wire master_if_waitrequest,

	input wire slave_if_clk,
	output wire slave_if_write,
	output wire slave_if_read,
	output wire [23:0] slave_if_address,
	output wire [31:0] slave_if_writedata,
	input wire[31:0] slave_if_readdata,
	input wire slave_if_waitrequest
   );

   wire master_reset_n;
   wire slave_reset_n;
   
 // _______________________________________________________________________________________________________________

  screset_sync master_reset_sync (.clk(master_if_clk),.data_in(1'b1), .reset_n(reset_n), .data_out(master_reset_n));
  screset_sync slave_reset_sync  (.clk(slave_if_clk), .data_in(1'b1), .reset_n(reset_n), .data_out(slave_reset_n));

 // _______________________________________________________________________________________________________________
 //			Master Clock Domain Logic
 // _______________________________________________________________________________________________________________

   reg ppln_master_read, ppln_master_write;
   reg async_operation_done;
   reg[2:0] ppln_operation_done;
   always@(posedge master_if_clk or negedge master_reset_n)
   begin
      if(!master_reset_n) 
	begin
	    ppln_master_write <= 1'b0;
	    ppln_master_read <= 1'b0;
	    ppln_operation_done <= 3'd0;
	end
      else 
	begin
	    ppln_master_write <= master_if_write;
	    ppln_master_read <= master_if_read;
	    ppln_operation_done <= {ppln_operation_done[1:0], async_operation_done};
	end
   end
 
  wire master_write_edge = master_if_blksel & master_if_write & ~ ppln_master_write; // posedge
  wire master_read_edge  = master_if_blksel & master_if_read &  ~ ppln_master_read; // posedge
  wire master_operation_done = ppln_operation_done[2]^ppln_operation_done[1];

 // __________________________________________________________________________________
 //			master fsm
 // __________________________________________________________________________________
  parameter IDLE = 2'd0, WRITE = 2'd1, READ = 2'd2;
  reg[1:0] mstate, next_mstate;

  always@(posedge master_if_clk or negedge master_reset_n)
   begin
      if(!master_reset_n) mstate <= IDLE;
      else mstate <= next_mstate;
   end 

 always@(*)
   begin
	next_mstate = mstate;
	case(mstate)
           IDLE: if (master_write_edge) next_mstate = WRITE;
           else if (master_read_edge) next_mstate = READ;

	   WRITE:if(master_operation_done) next_mstate = IDLE;
	   READ: if(master_operation_done) next_mstate = IDLE;
	   default: next_mstate = mstate;
  	endcase
   end

 // __________________________________________________________________
 //	master async interface signals

   reg async_if_write, async_if_read;
   always @(posedge master_if_clk or negedge master_reset_n)
       begin
	   if (!master_reset_n)
	       begin
		  async_if_read <= 1'b0;
		  async_if_write <= 1'b0;
	       end
	   else 
	       begin
		  if (master_read_edge) async_if_read  <= ~async_if_read;
		  if (master_write_edge) async_if_write  <= ~async_if_write;
	       end
       end

 // ___________________________________________________________________________________
 //		slave logic
 // ___________________________________________________________________________________

   reg[2:0] ppln_async_write, ppln_async_read;
   reg[31:0] ppln_writedata, ppln_address;
   always @(posedge slave_if_clk or negedge slave_reset_n)
       begin
	   if (!slave_reset_n)
	       begin
		  ppln_async_read[2:0]<= 3'd0;
		  ppln_async_write[2:0]<= 3'd0;
	       end
	   else 
	       begin
		  ppln_async_write[2:0]<= {ppln_async_write[1:0],async_if_write};
		  ppln_async_read[2:0] <= {ppln_async_read[1:0],async_if_read};
	       end
       end

  wire async_write_edge= ppln_async_write[1]^ppln_async_write[2];
  wire async_read_edge = ppln_async_read[1]^ppln_async_read[2];
  
 // __________________________________________________________________
 //		slave fsm
 // __________________________________________________________________

   reg[1:0] sstate, next_sstate;
   always @(posedge slave_if_clk or negedge slave_reset_n)
       begin
	   if (!slave_reset_n) sstate <= IDLE;
	   else  sstate <= next_sstate;
       end
  
 always@(*)
   begin
	next_sstate = sstate;
	case(sstate)
           IDLE: if (async_write_edge) next_sstate = WRITE;
           else if  (async_read_edge) next_sstate = READ;
	   WRITE:if(!slave_if_waitrequest) next_sstate = IDLE;
	   READ: if(!slave_if_waitrequest) next_sstate = IDLE;
	   default:next_sstate = sstate;
	endcase
   end

  wire slave_done = (sstate !== IDLE) && !slave_if_waitrequest;


  always @(posedge slave_if_clk or negedge slave_reset_n)
       begin
	   if (!slave_reset_n) async_operation_done <= 1'b0;
	   else if (slave_done) async_operation_done <= ~async_operation_done;
       end

  assign slave_if_write = (sstate == WRITE);
  assign slave_if_read  = (sstate == READ);
  assign slave_if_address = master_if_address[23:0];
  assign slave_if_writedata = master_if_writedata;

  assign master_if_waitrequest = master_if_blksel & ( master_write_edge||master_read_edge||(mstate == READ)||(mstate == WRITE));
  assign master_if_readdata_valid = (mstate == READ) && master_operation_done;

  //LKSIM commented assign master_if_readdata = slave_if_readdata;

  //LKSIM: intorduce simple mem to transfer data from slave to master
  //reason - avalon read data valid only 1 clk cycle and master readata
  //valid pulse is not align with the correct slave data


  wire master_ram_read = ppln_operation_done[1]^ppln_operation_done[0] && (mstate == READ);


  ram_dual #(.Address(1), .WordSize(32))
  master_slave_bridge_mem (.write_addr(1'b1), .read_addr(1'b1), .data(slave_if_readdata), .q(master_if_readdata),
	                   .we(slave_if_read && !slave_if_waitrequest), .oe(master_ram_read),
		           .write_clock(slave_if_clk), .read_clock(master_if_clk));





  

endmodule














// ____________________________________________________________________________________________
//	reset synchronizer 
// ____________________________________________________________________________________________

// turn off superfluous verilog processor warnings 
// altera message_level Level1 
// altera message_off 10034 10035 10036 10037 10230 10240 10030 

module screset_sync ( clk, data_in, reset_n, data_out) ;

  output data_out;
  input  clk;
  input  data_in;
  input  reset_n;

  reg   data_in_d1 /* synthesis ALTERA_ATTRIBUTE = "{-from \"*\"} CUT=ON ; PRESERVE_REGISTER=ON ; SUPPRESS_DA_RULE_INTERNAL=R101"  */;
  reg   data_out /* synthesis ALTERA_ATTRIBUTE = "PRESERVE_REGISTER=ON ; SUPPRESS_DA_RULE_INTERNAL=R101"  */;
  always @(posedge clk or negedge reset_n)
    begin
      if (reset_n == 0) data_in_d1 <= 0;
      else data_in_d1 <= data_in;
    end


  always @(posedge clk or negedge reset_n)
    begin
      if (reset_n == 0) data_out <= 0;
      else data_out <= data_in_d1;
    end



endmodule








// RAM Model

module ram_dual (data,read_addr,write_addr,oe,we,read_clock,write_clock,q);

parameter Address = 1; parameter WordSize = 32;

input [WordSize-1:0] data;
input [Address-1:0] read_addr, write_addr;
input oe, we, read_clock, write_clock;
output reg [WordSize-1:0] q;


// Declare the RAM variable
reg [WordSize-1:0] ram[0:1<<Address];


always @ (posedge write_clock)
begin
// Write
	if (we) ram[write_addr] <= data;
end

always @ (posedge read_clock)
begin
// Read 
	if (oe) q <= ram[read_addr];
	else q <= {WordSize{1'b0}};
end


endmodule
